package com.huimv.env.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.dao.entity.BaseWarningInfoEntity;
import com.huimv.env.dao.repo.BaseWarningInfoRepo;
import com.huimv.env.service.IAlarm;
import com.huimv.env.utils.Const;
import com.huimv.env.utils.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import javax.persistence.criteria.*;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
public class AlarmImpl implements IAlarm {
    @Autowired
    private BaseWarningInfoRepo warningInfoRepo;
    @Autowired
    private DateUtil dateUtil;

    @Override
    public Result getAlarm(Integer farmId, Integer alarmType, String alarmDateText, Integer pageNo, Integer pageSize) throws ParseException {
        //转换为日期才能比较
        Date alarmDate = dateUtil.formatDate(alarmDateText);
        Specification<BaseWarningInfoEntity> sf = (Specification<BaseWarningInfoEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != alarmType) {
                predList.add(criteriaBuilder.equal(root.get("alarmType").as(Integer.class), alarmType));
            }
            if (null != alarmDate) {
                predList.add(criteriaBuilder.equal(root.get("warningTime").as(Date.class), alarmDate));
            }
//            if (null != returnState) {
//                predList.add(criteriaBuilder.equal(root.get("returnState").as(Integer.class), returnState));
//            }
//            if (null != deliverState) {
//                predList.add(criteriaBuilder.equal(root.get("deliverState").as(Integer.class), deliverState));
//            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of( pageNo - 1, pageSize);
//        return new Result(ResultCode.SUCCESS,applyRepo.listApply(pageable,applyId));
        Page<BaseWarningInfoEntity>  warningInfoEntityPage = warningInfoRepo.findAll(sf, pageable);
        //格式化日期
//        List<BaseWarningInfoEntity> warningInfoEntityList = warningInfoEntityPage.getContent();
//        for(BaseWarningInfoEntity warningInfoEntity:warningInfoEntityList){
//            warningInfoEntity.setWarningTime(new Timestamp(dateUtil.formatDate(dateUtil.fromLongToDate(warningInfoEntity.getWarningTime().getTime(),"yyyy-MM-dd")).getTime()));
//            warningInfoEntity.setUploadTime(new Timestamp(dateUtil.formatDate(dateUtil.fromLongToDate(warningInfoEntity.getUploadTime().getTime(),"yyyy-MM-dd")).getTime()));
//        }
        return new Result(ResultCode.SUCCESS, warningInfoEntityPage);
    }

    @Override
    public Result getAlarm(Integer farmId, Integer alarmType, String alarmDateText,Integer quantity) throws ParseException {
        Specification<BaseWarningInfoEntity> sf = (Specification<BaseWarningInfoEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(Integer.class), farmId));
            }
            if (null != alarmType) {
                predList.add(criteriaBuilder.equal(root.get("alarmType").as(Integer.class), alarmType));
            }
            if (null != alarmDateText) {
                //转换为日期才能比较
                Date alarmDate = null;
                try {
                    alarmDate = dateUtil.formatDate(alarmDateText);
                } catch (ParseException e) {
                    e.printStackTrace();
                }
                predList.add(criteriaBuilder.equal(root.get("warningTime").as(Date.class), alarmDate));
            }
//            if (null != returnState) {
//                predList.add(criteriaBuilder.equal(root.get("returnState").as(Integer.class), returnState));
//            }
//            if (null != deliverState) {
//                predList.add(criteriaBuilder.equal(root.get("deliverState").as(Integer.class), deliverState));
//            }
            //
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
//        return new Result(ResultCode.SUCCESS,applyRepo.listApply(pageable,applyId));
        List<BaseWarningInfoEntity>  warningInfoEntityList = warningInfoRepo.findAll(sf);
        System.out.println("warningInfoEntityList.size="+warningInfoEntityList.size());
        //格式化日期
//        List<BaseWarningInfoEntity> warningInfoEntityList = warningInfoEntityPage.getContent();
//        for(BaseWarningInfoEntity warningInfoEntity:warningInfoEntityList){
//            warningInfoEntity.setWarningTime(new Timestamp(dateUtil.formatDate(dateUtil.fromLongToDate(warningInfoEntity.getWarningTime().getTime(),"yyyy-MM-dd")).getTime()));
//            warningInfoEntity.setUploadTime(new Timestamp(dateUtil.formatDate(dateUtil.fromLongToDate(warningInfoEntity.getUploadTime().getTime(),"yyyy-MM-dd")).getTime()));
//        }
        return new Result(ResultCode.SUCCESS, warningInfoEntityList);
    }

    /**
     * @Method      : getAlarm
     * @Description : 查询今天的警报
     * @Params      : [farmId, quantity]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/5       
     * @Time        : 14:54
     */
    @Override
    public Result getAlarm(Integer farmId, Integer quantity) throws ParseException {
        String today = dateUtil.getTodayDateText();
        //
        List<Object[]>  warningInfoEntityList = warningInfoRepo.findTodayAlarm(farmId,today,quantity);
        if(warningInfoEntityList.size()>0){
            JSONArray alarmJa = new JSONArray();
            for(int a=0;a<warningInfoEntityList.size();a++){
                Object[] alarmObj = (Object[]) warningInfoEntityList.get(a);
                JSONObject alarmJo = new JSONObject();
                alarmJa.add(alarmJo);
                alarmJo.put("levelName",alarmObj[0]);
                alarmJo.put("warningTime",alarmObj[1]);
                alarmJo.put("msg",alarmObj[2]);
            }
            return new Result(ResultCode.SUCCESS,alarmJa);
        }else{
            return new Result(Const.ERROR_CODE_NOTEXIST,Const.ERROR_MSG_NOTEXIST,false);
        }
    }

    /**
     * @Method      : getOneWeekAlarm
     * @Description : 报警级别分布
     * @Params      : [farmId]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/5       
     * @Time        : 17:02
     */
    @Override
    public Result getOneWeekAlarm(Integer farmId) throws ParseException {
        //倒序查询最近7天警报数据
        String startDate = dateUtil.getPastDate(6);
        String endDate = dateUtil.getTodayDateText();
        //
        List<BaseWarningInfoEntity> warningInfoEntityList = warningInfoRepo.findLastSevenDaysAlarm(farmId,startDate,endDate);
        JSONObject outJo = new JSONObject();
        outJo.put("one",_countAlarmTotal("一级",warningInfoEntityList));
        outJo.put("two",_countAlarmTotal("二级",warningInfoEntityList));
        outJo.put("three",_countAlarmTotal("三级",warningInfoEntityList));
        return new Result(ResultCode.SUCCESS,outJo);
    }

    //计算警报数量
    public Integer _countAlarmTotal(String typeName,List<BaseWarningInfoEntity> warningInfoEntityList){
        int count = 0;
        for(int a=0;a<warningInfoEntityList.size();a++){
            BaseWarningInfoEntity warningInfoEntity = warningInfoEntityList.get(a);
            if(warningInfoEntity.getLevelName().indexOf(typeName) != -1){
                count++;
//                warningInfoEntityList.remove(warningInfoEntity);
            }
        }
        return count;
    }

    @Override
    public Result getAlarmRate(Integer farmId) {
        return null;
    }
}
